/*
* Getting Started example sketch for nRF24L01+ radios
* This is a very basic example of how to send data from one node to another
* Updated: Dec 2014 by TMRh20
*/

#include <SPI.h>
#include "nRF24L01.h"
#include "RF24.h"

/****************** User Config ***************************/
/***      Set this radio as radio number 0 or 1         ***/
bool radioNumber = 0;

/* Hardware configuration: Set up nRF24L01 radio on SPI bus plus pins 7 & 8 */
RF24 radio(8, 10); // CSN, CE
/**********************************************************/

byte addresses[][6] = {"1ClkM","2ClkM"};

// Used to control whether this node is sending or receiving
bool role = 0;

void setup() {
  Serial.begin(19200);
  Serial.println(F("*** RECEIVE DUMP ****"));
  //Serial.println(F("*** PRESS 'T' to begin transmitting to the other node"));

  radio.begin();

  // Set the PA Level low to prevent power supply related issues since this is a
 // getting_started sketch, and the likelihood of close proximity of the devices. RF24_PA_MAX is default.
  radio.setPALevel(RF24_PA_LOW);
  radio.setChannel(1);
  radio.setDataRate(RF24_2MBPS);
  radio.setCRCLength(RF24_CRC_8);
  radio.setAutoAck(0);

  // Open a writing and reading pipe on each radio, with opposite addresses
  if (radioNumber) {
    radio.openWritingPipe(addresses[1]);
    radio.openReadingPipe(1,addresses[0]);
  } else {
    radio.openWritingPipe(addresses[0]);
    radio.openReadingPipe(1,addresses[1]);
  }
  radio.powerUp();
  Serial.println("Radio details:");
  radio.printDetails();

  // Start the radio listening for data
}

void sendRegistration(void) {
  char sendBuffer[32];

  memset(sendBuffer, 0, 32);
  sendBuffer[0] = 'r';
  strncpy(sendBuffer+1, "testClient", 10);
  radio.stopListening();
  radio.write(sendBuffer, 11);
  radio.startListening();
}

void loop() {
  char buffer[32];
  memset(buffer, 0, 32);

  if (radio.available()) {
    radio.read(buffer, 32);
    Serial.print(F("Received: type="));
    Serial.print(buffer[0]);
    if (buffer[0] == 'c') {
      Serial.print(F(", day="));
      Serial.print((int) buffer[1]);
      Serial.print(F(", time="));
      Serial.print((int) buffer[2]);
      Serial.print(':');
      Serial.print((int) buffer[3]);
      Serial.print(':');
      Serial.print((int) buffer[4]);
      Serial.print(F(", speed=1:"));
      Serial.println(1000.0 / ((int) buffer[5] << 2));
    } else if (buffer[0] == 'R') {
      Serial.print(F(", clockName="));
      Serial.println(buffer+1);
    } else {
      Serial.print(F("Unknown message received. Type="));
      Serial.print(buffer[0]);
      for (int i=0; i<8; ++i) {
        Serial.print(' '); Serial.print(buffer[i+1], HEX);
      }
      Serial.println(F(" ..."));
    }
 }

 static unsigned long lastTick = 0;
 if (millis() - lastTick > 5000) {
   lastTick = millis();
   Serial.print("#");
 }




#if 0
/****************** Ping Out Role ***************************/
if (role == 1)  {

    radio.stopListening();                                    // First, stop listening so we can talk.


    Serial.println(F("Now sending"));

    unsigned long start_time = micros();                             // Take the time, and send it.  This will block until complete
     if (!radio.write( &start_time, sizeof(unsigned long) )){
       Serial.println(F("failed"));
     }

    radio.startListening();                                    // Now, continue listening

    unsigned long started_waiting_at = micros();               // Set up a timeout period, get the current microseconds
    boolean timeout = false;                                   // Set up a variable to indicate if a response was received or not

    while ( ! radio.available() ){                             // While nothing is received
      if (micros() - started_waiting_at > 200000 ){            // If waited longer than 200ms, indicate timeout and exit while loop
          timeout = true;
          break;
      }
    }

    if ( timeout ){                                             // Describe the results
        Serial.println(F("Failed, response timed out."));
    }else{
        unsigned long got_time;                                 // Grab the response, compare, and send to debugging spew
        radio.read( &got_time, sizeof(unsigned long) );
        unsigned long end_time = micros();

        // Spew it
        Serial.print(F("Sent "));
        Serial.print(start_time);
        Serial.print(F(", Got response "));
        Serial.print(got_time);
        Serial.print(F(", Round-trip delay "));
        Serial.print(end_time-start_time);
        Serial.println(F(" microseconds"));
    }

    // Try again 1s later
    delay(1000);
  }



/****************** Pong Back Role ***************************/

  if ( role == 0 )
  {
    unsigned long got_time;

    if( radio.available()){
                                                                    // Variable for the received timestamp
      while (radio.available()) {                                   // While there is data ready
        radio.read( &got_time, sizeof(unsigned long) );             // Get the payload
      }

      radio.stopListening();                                        // First, stop listening so we can talk
      radio.write( &got_time, sizeof(unsigned long) );              // Send the final one back.
      radio.startListening();                                       // Now, resume listening so we catch the next packets.
      Serial.print(F("Sent response "));
      Serial.println(got_time);
   }
 }




/****************** Change Roles via Serial Commands ***************************/

  if ( Serial.available() )
  {
    char c = toupper(Serial.read());
    if ( c == 'T' && role == 0 ){
      Serial.println(F("*** CHANGING TO TRANSMIT ROLE -- PRESS 'R' TO SWITCH BACK"));
      role = 1;                  // Become the primary transmitter (ping out)

   }else
    if ( c == 'R' && role == 1 ){
      Serial.println(F("*** CHANGING TO RECEIVE ROLE -- PRESS 'T' TO SWITCH BACK"));
       role = 0;                // Become the primary receiver (pong back)
       radio.startListening();

    }
  }
#endif

} // Loop
